#ifndef SU2SPINOR_HXX
#define SU2SPINOR_HXX 1

#include "Geom3d/Geom3d.hxx"
#include "su2/su2mat.hxx"

#ifndef SU2SPINOR_FORMAT
#define SU2SPINOR_FORMAT "\t %11.8lf %+11.8lf i\n"
#endif
//------------------
class su2spinor
{
public:
  std::complex<double> p[2];

  // constructors
  su2spinor(const Point &v)
  {// This constructs a normalized spinor corresponding to the real 3-vector
   // v.  Such a spinor is not unique since there can be an arbitrary phase.

    Point n = normalize(v);
    if(n.z!=1.0)
      {
	double N = sqrt(2.0*(1.0-n.z));
	p[0] = (n.y+I*n.x)/N;
	p[1] = I*(1.0-n.z)/N;
      } else {
	p[0] = 1.0;
	p[1] = 0.0;
      }
  }
  // -- -- --
  su2spinor(const std::complex<double> &r, const std::complex<double> &i)
  {
    p[0] = r;
    p[1] = i;
  }
  // -- -- --
  su2spinor(void)
  {
    p[0] = 1.0; // pick a default.  This is spin up along the "z"-axis.
    p[1] = 0.0;
  }
  //---- ---- ---- ----
  friend su2spinor operator*(const su2mat M, const su2spinor psi)
  {//   = M  |ket> 
    su2spinor pd;
    pd.p[0] = M.m[0][0]*psi.p[0]+M.m[0][1]*psi.p[1];
    pd.p[1] = M.m[1][0]*psi.p[0]+M.m[1][1]*psi.p[1];
    return pd;
  }
  // -- -- --
  friend su2spinor operator*(const su2spinor psi, const su2mat M)
  {//  = <bra| M
    su2spinor pd;
    pd.p[0] = psi.p[0]*M.m[0][0]+psi.p[1]*M.m[1][0];
    pd.p[1] = psi.p[0]*M.m[0][1]+psi.p[1]*M.m[1][1];
    return pd;
  }
  // -- -- --
  friend std::complex<double> operator*(const su2spinor psi, const su2spinor phi)
  {//  = <bra| M
    std::complex<double> a;
    a = conj(psi.p[0])*phi.p[0]+conj(psi.p[1])*phi.p[1];
    return a;
  }
  // -- -- --
  su2spinor operator+(const su2spinor &psi)
  {
    su2spinor q;
    q.p[0] = p[0] + psi.p[0];
    q.p[1] = p[1] + psi.p[1];
    return q;
  }
  // -- -- --
  su2spinor operator-(const su2spinor &psi)
  {
    su2spinor q;
    q.p[0] = p[0] - psi.p[0];
    q.p[1] = p[1] - psi.p[1];
    return q;
  }
  su2spinor operator*(const std::complex<double> c) const
  {
    su2spinor q;
    q.p[0] = p[0]*c;
    q.p[1] = p[1]*c;
    return q;
  }
  // -- -- --
  friend su2spinor operator*(const std::complex<double> c, const su2spinor &pp)
  {
    su2spinor q;
    q = pp*c;
    return q;
  }
  // -- -- --
  friend su2spinor dagger(const su2spinor pc)
  {// Perhaps the <bra| and |ket> should be separate classes.
    su2spinor pd;
    pd.p[0] = conj(pc.p[0]);
    pd.p[1] = conj(pc.p[1]);
    return pd;
  }
  // -- -- --
  friend su2spinor normalize(const su2spinor pc)
  {//
    su2spinor pd;
    std::complex<double> n = sqrt(pc.p[0]*conj(pc.p[0])+pc.p[1]*conj(pc.p[1]));
    //    std::cout << "normalize su2spinor: n=" << n << "\n";

    if(abs(n)>0)
      {
	pd.p[0] = pc.p[0]/n;
	pd.p[1] = pc.p[1]/n;
      } else {
	std::cerr << "? class: su2spinor function: normalize";
	std::cerr << " could not normalize a null spinor\n";
      }
    return pd;
  }
  // -- -- --
  friend Point vector(const su2spinor pc)
  {
    Point v;
    v.x = 2*real(pc.p[0]*conj(pc.p[1]));
    v.y = -2*imag(pc.p[0]*conj(pc.p[1]));
    v.z = real(pc.p[0]*conj(pc.p[0])-pc.p[1]*conj(pc.p[1]));
    return v;
  }
  // -- -- --
  friend std::ostream& operator<<(std::ostream &os, const su2spinor &p)
  {
    int i, j;
    char numstr[50];
    for(i=0;i<2;i++)
      {
	sprintf(numstr, SU2SPINOR_FORMAT, real(p.p[i]), imag(p.p[i]));
	os << numstr;
      }
    return os;
  }
};

#endif
